export declare function buildContextString(context?: Record<string, unknown>): string;
/**
 * Formats a log message with context
 *
 * @param message - The message to format
 * @param args - Additional arguments for formatting
 * @returns The formatted message with context
 * @private
 */
export declare function formatMessage(displayContext: boolean, context: Record<string, unknown> | undefined, message: unknown, args: unknown[]): string;
//# sourceMappingURL=util.d.ts.map